/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 0.90B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "pv.h"

extern int timing, ticking;

FILE *fp;

int load_image( void *data )
{
	PtArg_t					args[2];
	PhDim_t					dim;
	PhPoint_t				pos;
	PhRect_t				rect;
	PtWorkProcId_t			*pid;
	PhImage_t				*nimage;
	char					buffer[PATH_MAX+5];
	int						changed = 0;

	data = data;

	pid = PtAppAddWorkProc( NULL, dumby_progress, 0 );

	if( NULL == ( nimage = PxLoadImage( file_name, &methods ) ) ) {
		PtAppRemoveWorkProc( NULL, pid );
//		if (!timing)
//			ApError( ABW_base, 2, "PV", "Unable to open graphics file", NULL );
		procid = NULL;
		if (timing) ticking = 1;
		return( Pt_HALT );
	}
	PtAppRemoveWorkProc( NULL, pid );
	if (timing) ticking = 1;

	sprintf( buffer,"%s",file_name );
	PtSetArg( &args[0], Pt_ARG_WINDOW_TITLE, buffer, 0 );
	PtSetResources( ABW_base, 1, args );

#if 0
fp = fopen("/home/pete/gifpal.pal", "w");
fwrite(nimage->palette, 256, 4, fp);
fclose(fp);
#endif

	/* free old image before starting */
	SetPalette( 0 );
	if( pimage ) {
		if( pimage->palette )
			memory_free( pimage->palette, PX_PALETTE );
		if( pimage->image )
			memory_free( pimage->image, PX_IMAGE );
	}
	pimage = nimage;

	update_file_list( );

	dim.w = pimage->size.w;
	dim.h = pimage->size.h + 33;

	if( dim.w < 300 )
		dim.w = 300;
	if( dim.h < 225 )
		dim.h = 225;
	PhWindowQueryVisible( Ph_QUERY_GRAPHICS, 0, 0, &rect );
	PtGetAbsPosition( ABW_base, &pos.x, &pos.y );
	pos.x = pos.x % ( rect.lr.x - rect.ul.x );
	pos.y = pos.y % ( rect.lr.y - rect.ul.y );
	if ( ( dim.w + pos.x + 6 ) > ( rect.lr.x - rect.ul.x ) ) {
		dim.w = (rect.lr.x - rect.ul.x) - pos.x - 6;
		changed = 1;
	}
	if ( ( dim.h + pos.y + 6 + (changed?17:0) ) > ( rect.lr.y - rect.ul.y ) ) {
		dim.h = (rect.lr.y - rect.ul.y) - pos.y - 6;
		changed += 2;
	}
	if ( changed == 2 )
		dim.w += 17;
	if (changed == 1 )
		dim.h += 17;		

	PtSetArg( &args[0], Pt_ARG_DIM, &dim, 0 );
	PtSetResources( ABW_base, 1, &args );
	PtSetArg( &args[0], Pt_ARG_DIM, &pimage->size, 0 );
	PtSetResources( RAW_image, 1, &args );
	PtDamageWidget( RAW_image );
    PtExtentWidgetFamily( ABW_picture_pane );
	procid = NULL;
	SetPalette( 1 );
	if (timing) ticking = 1;
	return( Pt_END );
}

void SetPalette( int mode )
{
	static int	cur_mode = 0;
	
	if( (PaletteType != Pg_PALSET_HARDINACTIVE) &&
		(PaletteType != Pg_PALSET_HARD))
		return;
	
	if ( mode && !has_focus ) {
		PaletteType = Pg_PALSET_HARDINACTIVE;
		cur_mode = 0;
		return;
	}
		
	if ( pimage && pimage->palette ) 
		PaletteType = (mode) ? Pg_PALSET_HARD : Pg_PALSET_HARDINACTIVE;

	if ( cur_mode == mode ) {
		return;
	} 
	
	if ( pimage && pimage->palette && (pimage->type & 0x10) ) {
		if ( PaletteType == Pg_PALSET_HARDINACTIVE ) {
			PgSetRegion( 1 );
			PgSetPalette( pimage->palette, pimage->palette_tag, 0, -1, Pg_PALSET_HARD, 0 );
			PgSetRegion( PtWidgetRid( ABW_base ) );
		} else
			PgSetPalette( pimage->palette, pimage->palette_tag, 0, pimage->colors, PaletteType, 0 );
	 	PtDamageWidget( ABW_picture_pane );
	}
	cur_mode = mode;
}

int dumby_progress( void *data )
{
	data = data;

	return( Pt_CONTINUE );
}

update_file_list( )
{
	int cnt;
	
	for( cnt = 1; cnt < 4; cnt++ ) {
		if( strcmp( file_name, &file_list[cnt][0] ) == 0 )
			return;
	}

	if( file_list_cnt < 3 ) {
		file_list_cnt++;
		strcpy( &file_list[file_list_cnt][0], file_name );
	}
	else {
		strcpy( &file_list[3][0], &file_list[2][0] );
		strcpy( &file_list[2][0], &file_list[1][0] );
		strcpy( &file_list[1][0], file_name );
	}
}

void RAW_image_draw( PtWidget_t * widget, PhTile_t *damage )
{
	PhRect_t	rect;
	
	PtBasicWidgetCanvas( widget, &rect );
	
	widget = widget, damage = damage;
	
	if( pimage )
	{
		if( ( pimage->type & 0x10 ) && pimage->palette && PaletteType != Pg_PALSET_HARDINACTIVE )
			PgSetPalette( pimage->palette, pimage->palette_tag, 0, pimage->colors, PaletteType, 0 );

		if( pimage->type == 0xff ) {
			PgSetTextColor( pimage->palette[1] );
			PgSetFillColor( pimage->palette[0] );
			PgDrawBitmap( pimage->image, Pg_BACK_FILL, &rect.ul, (PhPoint_t *)&pimage->size, pimage->bpl, 0 );
			}
		else
			PgDrawImagemx( pimage->image, pimage->type, &rect.ul, (PhPoint_t *)&pimage->size, pimage->bpl, pimage->image_tag );
	}
}