/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 0.90B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "pv.h"
#include "abimport.h"
#include "proto.h"

PhImage_t		*pimage = NULL;
char			file_path[PATH_MAX + 2];
char			file_name[PATH_MAX];
PxMethods_t		methods;
//PgColor_t		default_palette[256];
PtWorkProcId_t	*procid;

extern int get_palette;
extern int pv_quantize;
unsigned long palette[256];

int
pv_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PhRect_t				rect = { 0, 0, 0, 0 };
	PtArg_t					args[5];
	PhPoint_t				pos = {0,0};

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	getcwd( file_path, PATH_MAX + 1 );
	if (file_path == NULL || file_path[0] == '\0')
		strcpy(file_path,"/usr/photon/");
		
//	dflt_palette( );
	PhEventArm( );

	memset( &methods, 0, sizeof( methods ) );

// | PX_QUANTITIZE;

	methods.px_alloc = memory_allocate;
	methods.px_free = memory_free;
	methods.px_progress = progress;
	methods.px_error = error_hdlr;

	if (pv_quantize)
	  {
		if (get_palette)
		  {
			methods.flags |= PX_LOAD | PX_DODITHER;
		  }
		else
		  {
			methods.flags |= PX_LOAD | PX_USECOLORS | PX_DODITHER;
			methods.colors = palette;
			methods.ncolors = 256;
		  }
	  }
	else
	  {
		methods.flags |= PX_LOAD | PX_DIRECT_COLOR;
	  }

	/* increase the draw buffer size */
	PgSetDrawBufferSize( 64000 );

	PtSetArg( &args[0], Pt_ARG_RAW_DRAW_F, RAW_image_draw, 0 );
	PtSetArg( &args[1], Pt_ARG_POS, &pos, 0 );	
	PtSetArg( &args[2], Pt_ARG_FILL_COLOR, Pg_GREY, 0 );	
	PtSetArg( &args[3], Pt_ARG_BORDER_WIDTH, 0, 0 );
	RAW_image = PtCreateWidget( PtRaw, ABW_picture_pane, 4, args );

	if( file_name[0] != '\0' )
		procid = PtAppAddWorkProc( NULL, load_image, 0 );

	return( Pt_CONTINUE );
}

void * memory_allocate( long nbytes, int type )
{
	if( type == PX_IMAGE && UseShmem )
		return( PgShmemCreate( nbytes+512, NULL ) );
	else
		return( calloc( 1, nbytes+512 ) );
}

void memory_free( void *memory, int type )
{
	if( type == PX_IMAGE && UseShmem )
		PgShmemDestroy( memory );
	else
		free( memory );
}

void progress( int percent )
{               	
	static int old_percent;
	char	buffer[PATH_MAX+5];
	PtArg_t	args[1];
	
	percent = percent >> 16;

	if ( percent < 10 )
		old_percent = 0;
	if ( ( percent - old_percent ) >= 10 ) {
		sprintf( buffer,"loading... %s %d%%",file_name, percent );
		PtSetArg( &args[0], Pt_ARG_WINDOW_TITLE, buffer, 0 );
		PtSetResources( ABW_base, 1, args );
		old_percent = percent;
	}
	PtProcessEvent( );
}

void error_hdlr( char *msg )
{
//	ApError( ABW_base, 2, "PV", msg, NULL );
	PtAskQuestion( ABW_base, "PV", msg, NULL, "OK", NULL, NULL, 1 );
}